
using GPMaxlik, NLopt

# Simulate some data to fit. This data also generates the saa vectors, saav.
include("simulate_data.jl")

# Choose the max number of iterations for using stochastic derivatives:
GPMaxlik.SETTINGS.STOCH_MAXIT = 10

# Create the objective function in NLopt format.
const counter = [0]
function objective(p, g, saa_input)
  global counter[1] += 1
  if iszero(rem(counter[], 5)) 
    println("Iter $(counter[]): $(round.(p, digits=4))")
  end
  nll_call = gnll(pts, simul, kfn, (kd1, kd2), p, saa=saa_input, 
                  nll=true, grad=length(g)>0, fish=false, vrb=true)
  if length(g) > 0
    g .= nll_call.grad
  end
  nll_call.nll
end

# Compare exact and stochastic estimates.
for (method, saa) in ((:stochastic, saav), (:exact, nothing))

  # Reset the iteration counter and stochastic derivative flags.
  println()
  GPMaxlik.reset_settings!()
  counter[1] = 0

  # set up the problem.
  opt = Opt(:LD_MMA, length(pr))
  min_objective!(opt, (p,g)->objective(p,g,saa))
  ftol_rel!(opt, 1.0e-6)

  # Solve the problem.
  println("Timing for $method estimation:")
  @time (minf, mle, ret) = optimize(opt, [1.0, 5.0])
  if ret != :FTOL_REACHED
    println("Optimization for $method estimation failed: return $ret")
    continue
  end
  println("Number of iterations: $(counter[])")

  # Get the fisher matrix of the estimate.
  fish = gnll(pts, simul, kfn, (kd1, kd2), mle, saa=saa, 
             nll=false, grad=false, fish=true).fish

  # Print output.
  println("Result for $method estimation:")
  for (v, pm) in zip(mle, sqrt.(diag(inv(fish))))
    println(round(v, digits=6), " ± ", round(pm*1.96, digits=6))
  end

end

