
using GPMaxlik

# Simulate some data to fit. This file also generates the saa vectors, saav.
include("simulate_data_anisotropy.jl")

# Temporarily, because of the strategy for memory buffer reuse, you need to
# provide an EVEN number of derivative matrix buffers. This model only has 7
# parameters, so this is kind of annoying.
const K   = GPMaxlik.build_covmat(pts, kfn, pr)
const Kdv = [GPMaxlik.build_dcovmat(pts) for _ in 1:8]

# No prealloc:
function nll_grad_fish_none(p)
  nll_call = gnll(pts, simul, kfn, dkfunv, p, saa=nothing,
                 nll=true, grad=true, fish=true, vrb=true)
  (nll_call.nll, nll_call.grad, nll_call.fish)
end

# Partial prealloc:
function nll_grad_fish_some(p)
  nll_call = gnll(pts, simul, kfn, dkfunv, p, saa=nothing,
                 nll=true, grad=true, fish=true, vrb=true,
                 K=K, Kdv=Kdv[1:4])
  (nll_call.nll, nll_call.grad, nll_call.fish)
end

# More partial prealloc:
function nll_grad_fish_more(p)
  nll_call = gnll(pts, simul, kfn, dkfunv, p, saa=nothing,
                 nll=true, grad=true, fish=true, vrb=true,
                 K=K, Kdv=Kdv[1:6])
  (nll_call.nll, nll_call.grad, nll_call.fish)
end

# Full prealloc:
function nll_grad_fish_full(p)
  nll_call = gnll(pts, simul, kfn, dkfunv, p, saa=nothing,
                 nll=true, grad=true, fish=true, vrb=true,
                 K=K, Kdv=Kdv)
  (nll_call.nll, nll_call.grad, nll_call.fish)
end

println("\nTiming with no prealloc:")
nllgf_none = nll_grad_fish_none(pr) # for precompilation
@timev nllgf_none = nll_grad_fish_none(pr)
print("\n\n\n")

println("Timing with 4/7 prealloc:")
nllgf_some = nll_grad_fish_some(pr) # for precompilation
@timev nllgf_some = nll_grad_fish_some(pr)
print("\n\n\n")

println("Timing with 6/7 prealloc:")
nllgf_more = nll_grad_fish_more(pr) # for precompilation
@timev nllgf_more = nll_grad_fish_more(pr)
print("\n\n\n")

println("Timing with full prealloc:")
nllgf_full = nll_grad_fish_full(pr) # for precompilation
@timev nllgf_full = nll_grad_fish_full(pr)
print("\n")

print("Checking for numerical agreement...")
@assert nllgf_none[2] == nllgf_some[2] == nllgf_more[2] == nllgf_full[2] "ERROR: Gradients disagree."
@assert nllgf_none[3] == nllgf_some[3] == nllgf_more[3] == nllgf_full[3] "ERROR: Fisher matrices disagree."
println("all derivatives agree. Looks good.")

