module GPMaxlik

  using Statistics, LinearAlgebra

  export gnll, trustregion

  # An Inf for integers, which just promises to be greater than any Int.
  struct InfiniteInt end
  Base.isless(x::Int64, InfInt::InfiniteInt) = true
  Base.isless(InfInt::InfiniteInt, x::Int64) = false

  mutable struct Settings
    STOCH_MAXIT::Union{InfiniteInt, Int64}
    COUNTER::Int64
    GRAD_INFNORM_TOL::Float64
    FISH_INFNORM_TOL::Float64
    STOCH_GRAD::Bool
    STOCH_FISH::Bool
  end

  const SETTINGS = Settings(InfiniteInt(), 0, 0.0, 0.0, true, true)

  function reset_settings!(;grad_tol=nothing, fish_tol=nothing, maxit=nothing)
    SETTINGS.STOCH_GRAD=true
    SETTINGS.STOCH_FISH=true
    if !isnothing(grad_tol)
      SETTINGS.GRAD_INFNORM_TOL = grad_tol
    end
    if !isnothing(fish_tol)
      SETTINGS.FISH_INFNORM_TOL = fish_tol
    end
    if !isnothing(maxit)
      SETTINGS.STOCH_MAXIT = maxit
    end
    SETTINGS.COUNTER = 0
    nothing
  end

  include("util.jl")

  include("covmatrix.jl")

  include("nll.jl")

  include("interfaces.jl")

  include("trustregion.jl")

end
